==> jview-0.1.2.tar.gz: jview-0.1.2/ldb.h <==
/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LDB_H__
#define __LDB_H__

#include <config.h>
#include <stdio.h>
#include <netinet/in.h>


#define GINT8( ptr) ( *((gint8*) ptr))
#define GINT16( ptr) ( *((gint16*) ptr))
#define GINT32( ptr) ( *((gint32*) ptr))
#define GUINT8( ptr) ( *((guint8*) ptr))
#define GUINT16( ptr) ( *((guint16*) ptr))
#define GUINT32( ptr) ( *((guint32*) ptr))

/*
  these are macros to read big endians
  */
#ifdef WORDS_BIGENDIAN
#define GUINT16BE( n) n
#define GUINT32BE( n) n
#else
#define GUINT16BE( n) ntohs( n)
#define GUINT32BE( n) ntohl( n)
#endif

typedef struct _LDB LDB;
typedef struct _LDBTypeInfo LDBTypeInfo;
typedef struct _LDBTypeMethods LDBTypeMethods;
typedef struct _LDBHeader LDBHeader;
typedef struct _LDBBlock LDBBlock;
typedef struct _LDBBlockHeader LDBBlockHeader;
typedef struct _LDBCursor LDBCursor;
typedef struct _LDBItemId LDBItemId;
typedef struct _LDBItem LDBItem;
typedef gint     (*LDBRead)   ( LDB* ldb, LDBCursor *cursor, LDBItem *item, void *read_data);
typedef gint     (*LDBWrite)  ( LDB* ldb, LDBCursor *cursor, LDBItem *item, void *write_data);

typedef enum {
    LDB_BLOCK_MODIFIED     = 1 << 2,
    LDB_BLOCK_CACHE        = 1 << 3,
    LDB_BLOCK_ALREADY_READ = 1 << 4
} LDBBlockFlags;

typedef enum {
    LDB_READ,
    LDB_WRITE
} LDBCursorMode;

struct _LDBHeader {
    guint16 types;
    guint16 version;
    guint16 block_size;
    guint32 block_count;
    LDBTypeInfo *type_info;
};

struct _LDB {
    FILE *file;
    gulong start_pos;
    gulong first_block_pos;
    LDBHeader header;
   
    GArray *blocks;

    LDBTypeMethods *methods;
};


struct _LDBTypeMethods {
    LDBRead read;
    void     *read_data;
    LDBWrite write;
    void     *write_data;
};

struct _LDBTypeInfo {
    guint32 first_block;
    guint32 last_block;
    guint32 block_count;
    guint32 item_count;
};

struct _LDBBlockHeader {
    guint16 free_space;
    guint32 prev_block;
    guint16 next_item_id;
};

struct _LDBBlock {
    guint32 block_no;
    guint16 flags;
    GSList  *deleted_items;
    LDBBlockHeader header;
    char *data;
};


struct _LDBCursor {
    guint16 type;

    LDBBlock *block;

    LDBCursorMode mode;
   
    guint16 item_no;
    char *pos;
    char *private_pos;
};

struct _LDBItemId {
    guint32 block_no;
    guint16 item_no;
};

struct _LDBItem {
    LDBItemId id;
    guint16 size;
    void  *data;
};


LDB*        ldb_create( FILE *file, 
			guint block_size,
			gint  types);
LDB*        ldb_load( FILE *file);
void        ldb_close( LDB *ldb);
gint        ldb_flush( LDB *ldb);

LDBBlock*   ldb_new_block( LDB *ldb, guint16 type);
LDBBlock*   ldb_read_block( LDB *ldb, guint32 block_no);
gint        ldb_write_block( LDB *ldb, LDBBlock *block);
void        ldb_release_block( LDB *ldb, LDBBlock *block);

void        ldb_set_read( LDB *ldb, gint type, LDBRead read, void *read_data);
void        ldb_set_write( LDB *ldb, gint type, LDBWrite write, void *write_data);

LDBCursor*  ldb_cursor_new( LDB *ldb, guint16 type, LDBCursorMode mode);
void        ldb_cursor_free( LDB *ldb, LDBCursor *cursor);
gint        ldb_read_next_item( LDB *ldb, LDBCursor *cursor, LDBItem *item_buff);
gint        ldb_write_item( LDB *ldb, LDBCursor *cursor, LDBItem *item);
gint        ldb_remove_item( LDB *ldb, LDBItemId id);

gint        ldb_dump( LDB *ldb);


#endif /* __LDB_H__ */

==> jview-0.1.3.tar.gz: jview-0.1.3/ldb.h <==
/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __LDB_H__
#define __LDB_H__

#include <config.h>
#include <stdio.h>
#include <netinet/in.h>


#define GINT8( ptr) ( *((gint8*) ptr))
#define GINT16( ptr) ( *((gint16*) ptr))
#define GINT32( ptr) ( *((gint32*) ptr))
#define GUINT8( ptr) ( *((guint8*) ptr))
#define GUINT16( ptr) ( *((guint16*) ptr))
#define GUINT32( ptr) ( *((guint32*) ptr))

/*
  these are macros to read big endians
  */
#ifdef WORDS_BIGENDIAN
#define GUINT16BE( n) n
#define GUINT32BE( n) n
#else
#define GUINT16BE( n) ntohs( n)
#define GUINT32BE( n) ntohl( n)
#endif

#define LDB_VERSION 1
#define LDB_MARKER "LDB0"
#define LDB_MARKER_SIZE 4

typedef struct _LDB LDB;
typedef struct _LDBTypeInfo LDBTypeInfo;
typedef struct _LDBTypeMethods LDBTypeMethods;
typedef struct _LDBHeader LDBHeader;
typedef struct _LDBBlock LDBBlock;
typedef struct _LDBBlockHeader LDBBlockHeader;
typedef struct _LDBCursor LDBCursor;
typedef struct _LDBItemId LDBItemId;
typedef struct _LDBItem LDBItem;
typedef gint     (*LDBRead)   ( LDB* ldb, LDBCursor *cursor, LDBItem *item, void *read_data);
typedef gint     (*LDBWrite)  ( LDB* ldb, LDBCursor *cursor, LDBItem *item, void *write_data);

typedef enum {
    LDB_BLOCK_MODIFIED     = 1 << 2,
    LDB_BLOCK_CACHE        = 1 << 3,
    LDB_BLOCK_ALREADY_READ = 1 << 4
} LDBBlockFlags;

typedef enum {
    LDB_READ,
    LDB_WRITE
} LDBCursorMode;

#define LDB_HEADER_SIZE 12 + LDB_MARKER_SIZE
struct _LDBHeader {
    guint16 user_header_size;
    guint16 types;
    guint16 version;
    guint16 block_size;
    guint32 block_count;
    char    marker[ LDB_MARKER_SIZE];
    LDBTypeInfo *type_info;
};

struct _LDB {
    gint fd;
    glong first_block_pos;
    LDBHeader header;
   
    GArray *blocks;

    LDBTypeMethods *methods;
};


struct _LDBTypeMethods {
    LDBRead read;
    void     *read_data;
    LDBWrite write;
    void     *write_data;
};

#define LDB_TYPE_INFO_SIZE 16
struct _LDBTypeInfo {
    guint32 first_block;
    guint32 last_block;
    guint32 block_count;
    guint32 item_count;
};

struct _LDBBlockHeader {
    guint16 free_space;
    guint32 prev_block;
    guint16 next_item_id;
};

struct _LDBBlock {
    guint32 block_no;
    guint16 flags;
    GSList  *deleted_items;
    LDBBlockHeader header;
    char *data;
};


struct _LDBCursor {
    guint16 type;

    LDBBlock *block;

    LDBCursorMode mode;
   
    guint16 item_no;
    char *pos;
    char *private_pos;
};

struct _LDBItemId {
    guint32 block_no;
    guint16 item_no;
};

struct _LDBItem {
    LDBItemId id;
    guint16 size;
    void  *data;
};


LDB*        ldb_create( char *path, 
			guint block_size,
			gint  types,
			gint header_size);
LDB*        ldb_load( char *path, gint header_size);
void        ldb_close( LDB *ldb);
gint        ldb_flush( LDB *ldb);
glong       ldb_get_user_header_position( LDB *ldb);

LDBBlock*   ldb_new_block( LDB *ldb, guint16 type);
LDBBlock*   ldb_read_block( LDB *ldb, guint32 block_no);
gint        ldb_write_block( LDB *ldb, LDBBlock *block);
void        ldb_release_block( LDB *ldb, LDBBlock *block);

void        ldb_set_read( LDB *ldb, gint type, LDBRead read, void *read_data);
void        ldb_set_write( LDB *ldb, gint type, LDBWrite write, void *write_data);

LDBCursor*  ldb_cursor_new( LDB *ldb, guint16 type, LDBCursorMode mode);
void        ldb_cursor_free( LDB *ldb, LDBCursor *cursor);
gint        ldb_read_next_item( LDB *ldb, LDBCursor *cursor, LDBItem *item_buff);
gint        ldb_write_item( LDB *ldb, LDBCursor *cursor, LDBItem *item);
gint        ldb_remove_item( LDB *ldb, LDBItemId id);

gint        ldb_dump( LDB *ldb);


#endif /* __LDB_H__ */
